' select MCU and clock speed
Device = 18F4550
XTAL = 48	
OPTIMISER_LEVEL = 6
Declare SHIFT_DELAYUS 0

' descriptor file, located in \inc\usb_18 - a copy
' is located in the same folder as this file	
USB_DESCRIPTOR = "Glyphix_18F4550DESC.Inc"
'USB_DESCRIPTOR = "CDCDESC.inc"

' USB Buffer...
Symbol USBBufferSizeMax = 64
Symbol USBBufferSizeTX  = 64
Symbol USBBufferSizeRX  = 64
Dim    USBBuffer[USBBufferSizeMax] As Byte

' some useful flags...
Dim PP0 As Byte SYSTEM        ' USBPOLL status return
Symbol CARRY_FLAG = STATUS.0  ' high if microcontroller does not have control over the buffer
Symbol ATTACHED_STATE = 6     ' is USB attached

DelayMS 1000
ALL_DIGITAL = true
'-----------------------------------------------------------------------
'--- LCD related                                                     ---
'-----------------------------------------------------------------------
        Declare LCD_DTPORT  PORTB		'use B0-B7 	(PIC) for D0-D7 	(LCD)
        Declare LCD_RSPIN   PORTD.5     ' = D/I pin op LCD !
        Declare LCD_ENPIN   PORTD.7
        Declare LCD_RWPIN   PORTD.6
        Declare LCD_CS1PIN  PORTA.3
        Declare LCD_CS2PIN  PORTA.5
		Declare LCD_TYPE 1				'1 = Samsung KS-compatible 128x64 pixels
        Declare INTERNAL_FONT 1  
        Declare GLCD_CS_INVERT 1
        Declare GLCD_STROBE_DELAY 1
        Declare GLCD_READ_DELAY 1

'-----------------------------------------------------------------------
'--- KEYPAD related                                                  ---
'-----------------------------------------------------------------------
Output PORTE.0
Output PORTE.1
Output PORTE.2
Output PORTD.2
Input PORTD.3
Input PORTC.6
Input PORTC.7
Input PORTD.4
Symbol Row1 = PORTE.0
Symbol Row2 = PORTE.1
Symbol Row3 = PORTE.2
Symbol Row4 = PORTD.2
Symbol Col1 = PORTD.3
Symbol Col2 = PORTC.6
Symbol Col3 = PORTC.7
Symbol Col4 = PORTD.4
Dim Btn1 As Bit
Dim Btn2 As Bit
Dim Btn3 As Bit
Dim Btn4 As Bit
Dim Btn5 As Bit
Dim Btn6 As Bit
Dim Btn7 As Bit
Dim Btn8 As Bit
Dim Btn9 As Bit
Dim Btn0 As Bit
Dim BtnA As Bit
Dim BtnB As Bit
Dim BtnC As Bit
Dim BtnD As Bit
Dim BtnCross As Bit
Dim BtnStar As Bit
Dim BytePos As Byte

'-----------------------------------------------------------------------
'--- RTC Clock DS1302 declaration                                    ---
'-----------------------------------------------------------------------
Symbol SCLK	= PORTA.0
Symbol IO	= PORTA.1
Symbol RST  = PORTA.2
Dim RtcYear         As Byte
Dim RtcDay          As Byte
Dim RtcMonth        As Byte
Dim RtcDate         As Byte
Dim RtcHr           As Byte
Dim RtcMin          As Byte
Dim RtcSec          As Byte
Dim RtcControl      As Byte
Dim SecondsMemory   As Byte
Dim TmpSeconds      As Byte

'--- variables that will hold comparison values for scheduling ---
Dim SchYear         As Byte
Dim SchDay          As Byte
Dim SchMonth        As Byte
Dim SchHr           As Byte
Dim SchMin          As Byte
Dim TmpCharYear     As String * 2
Dim TmpCharMonth    As String * 2
Dim TmpCharDay      As String * 2
Dim TmpCharHr       As String * 2
Dim TmpCharMin      As String * 2

'-----------------------------------------------------------------------
'--- menu related                                                    ---
'-----------------------------------------------------------------------
Dim MnuTitle As String * 17
Dim MnuA As Bit
Dim MnuB As Bit
Dim MnuC As Bit
Dim MnuD As Bit
Dim MnuStar As Bit
Dim MnuCross As Bit
Dim MnuStyle As Byte
Dim MnuType As Byte
Dim MnuItem1 As String * 14
Dim MnuItem2 As String * 14
Dim mnuItem3 As String * 14
Dim MnuItem4 As String * 14
Dim MnuItem5 As String * 14
Dim MnuInv1 As Bit
Dim MnuInv2 As Bit
Dim MnuInv3 As Bit
Dim MnuInv4 As Bit
Dim MnuInv5 As Bit
Dim MnuSelect As Byte
Dim MnuMaxPos As Byte

'-----------------------------------------------------------------------
'--- internal variables                                              ---
'-----------------------------------------------------------------------
Dim PosVal As string*14
Dim Xpos As Byte
Dim Ypos As Byte
Dim X As Word
Dim y As Byte
Dim Z As Byte
Dim TmpByte As Byte
Dim BitInverse As Bit
Dim BitExit As Bit

Dim BitScheduler As Bit
Dim BitUSB As Bit
Dim BitExternal As Bit
Dim BitBacklight As Bit
Dim BitPlayList As Bit
Dim BitAnimation As Bit
Dim ANILine As String * 17
Dim LSTLine As String * 17
Dim SCHLine As String * 17

'-----------------------------------------------------------------------
'--- sending and receiving data across both PIC's                    ---
'-----------------------------------------------------------------------
Dim Dout As string*17
Dim Din As string*17
Dim xxx As Byte
Dim yyy As Byte
Dim zzz As Byte

Input PORTC.0
Input PORTC.1
Output  PORTD.0
Output  PORTD.1
Low PORTD.0
Low PORTD.1

Symbol DOutPin = PORTD.0
Symbol DOutClk = PORTD.1
Symbol DInPin  = PORTC.0
Symbol DInClk  = PORTC.1

'-----------------------------------------------------------------------
'--- initialisation                                                  ---        
'-----------------------------------------------------------------------
'--- LCD startup screen ---
Cls
X = 0
For Ypos = 0 To 7
    For Xpos = 0 To 127
        Z = LRead StartupScreen + X
        LCDWrite Ypos,Xpos,[Z]
        X = X + 1
    Next Xpos
Next Ypos
'--- do init of clock ---
GoSub InitClock
'--- set rows for keypad low ---
Low Row1
Low Row2
Low Row3
Low Row4
'--- read setup from internal EEPROM ---
GoSub ReadSetupFromEEPRom
GoSub SendConfigBits
X=0
Cls
BitAnimation = 0
BitPlayList = 0
LSTLine = "--------  ---/---"
ANILine = "--------  (-----)"
SCHLine = "--/--/----  --:--"
GoSub PaintIconsOnMainScreen
'-----------------------------------------------------------------------
'--- main application                                                ---        
'-----------------------------------------------------------------------

Mainloop:
    
   GoSub GetTime
   If TmpSeconds <> RtcSec Then
      TmpSeconds = RtcSec
      Print At 0,0,Inverse 1,"GLYPHIX      "
      Print At 0,13,Inverse 1,HEX2 RtcHr,":",HEX2 RtcMin,":",HEX2 RtcSec
      Print At 7,11,Inverse 1,HEX2 RtcDay,"/",HEX2 RtcMonth,"/20",HEX2 RtcYear
      Select Case RtcDate
        Case 1
            Print At 7,0,Inverse 1,"Maandag    "    
        Case 2
            Print At 7,0,Inverse 1,"Dinsdag    "        
        Case 3
            Print At 7,0,Inverse 1,"Woensdag   "        
        Case 4
            Print At 7,0,Inverse 1,"Donderdag  "        
        Case 5
            Print At 7,0,Inverse 1,"Vrijdag    "        
        Case 6
            Print At 7,0,Inverse 1,"Zaterdag   "   
        Case 7
            Print At 7,0,Inverse 1,"Zondag     "        
      End Select
   EndIf
   GoSub ScanKeyPad
   If BtnCross = 1 Or BtnStar = 1 Then
      GoSub DoMenu
   EndIf
   '--- only check for scheduling-event, when scheduling is enabled ---
   If BitScheduler = 1 Then
      If SchYear = 0 And SchMonth = 0 And SchDay = 0 And SchHr = 0 And SchMin = 0 Then
        Dout[0] = "C"
        Dout[1] = "W"
        GoSub AddDateTimeToDout
        GoSub SendData  '--- request for scheduled date/time ---
        DelayUS 10
        Repeat Until DInPin = 1
        GoSub ReceiveData
        GoSub ParseData
      EndIf
      If SchYear = RtcYear And SchMonth = RtcMonth And SchDay = RtcDay And SchHr = RtcHr And SchMin = RtcMin Then
        '--- send notice to main controller, that it's time to start something ---
        Dout[0] = "T"
        Dout[10] = SchYear
        Dout[11] = SchMonth
        Dout[12] = SchDay
        Dout[13] = SchHr
        Dout[14] = SchMin
        GoSub SendData
        DelayUS 50
        '--- clear variables, otherwise it would be triggered more then once ---
        SchYear = 0:SchMonth = 0:SchDay = 0:SchHr = 0:SchMin = 0
      EndIf
   EndIf
   '--- check if we receive data from maincontroller  ---
   '--- mainly to update SCHline, ANIline and LSTline ---
   If DInPin = 1 Then
      GoSub ReceiveData
      GoSub ParseData
    EndIf

GoTo Mainloop       


'-----------------------------------------------------------------------
'--- do something with the received data                             ---
'-----------------------------------------------------------------------
ParseData:
      Select Case Din[0]
         Case "W"   '--- update watchdog-variables for scheduler (e.g. SCHyear,SCHmonth etc ...) ---
            If Din[1] ="1" Then
                TmpCharYear  = Mid$(Din,7,2) :SchYear  = Val(TmpCharYear ,HEX)
                TmpCharMonth = Mid$(Din,5,2) :SchMonth = Val(TmpCharMonth,HEX)
                TmpCharDay   = Mid$(Din,3,2) :SchDay   = Val(TmpCharDay  ,HEX)
                TmpCharHr    = Mid$(Din,9,2) :SchHr    = Val(TmpCharHr   ,HEX)
                TmpCharMin   = Mid$(Din,11,2):SchMin   = Val(TmpCharMin  ,HEX)
                SCHLine = TmpCharDay + "/"+ TmpCharMonth + "/20" + TmpCharYear + "  " + TmpCharHr + ":" + TmpCharMin
                GoSub PaintIconsOnMainScreen
            Else
                SchYear = 99:SchMonth = 99:SchDay=99:SchHr=99:SchMin = 99
                SCHLine = "--/--/----  --:--"
                GoSub PaintIconsOnMainScreen
            EndIf
         Case "L"   '--- update LST line ---
            LSTLine = Mid$(Din,3,8)+ "  " + Mid$(Din,11,3) + "/" + Mid$(Din,14,3)
            GoSub SetAnimationOrPlayListBit         
         Case "A"   '--- update ANI line ---
            ANILine = Mid$(Din,3,8) + "  (" + Mid$(Din,11,5) + ")"     
            GoSub SetAnimationOrPlayListBit         
      End Select
Return

'-----------------------------------------------------------------------
'--- if din[1] with L or A = 1, then we are playing a playlist       ---
'---- otherwise, we are playing an animation                         ---
'-----------------------------------------------------------------------
SetAnimationOrPlayListBit:
    If Din[1] = "1" Then
        BitPlayList = 1
        BitAnimation = 0
    EndIf
    GoSub PaintIconsOnMainScreen
Return

'-----------------------------------------------------------------------
'--- show icons on mainscreen, accordinly if bits are set or not     ---
'-----------------------------------------------------------------------
PaintIconsOnMainScreen:
    GoSub ClearIcons:
    X=0
    For Ypos = 1 To 3
        For Xpos = 3 To 26
        If BitAnimation = 1 Then
            Z = LRead Icon_ANI + X
            LCDWrite Ypos,Xpos + 0,[Z]
        EndIf
        If BitPlayList = 1 Then
            Z = LRead Icon_LST + X
            LCDWrite Ypos,Xpos + 0,[Z]
        EndIf
        If BitScheduler = 1 Then
            Z = LRead Icon_Scheduler + X
            LCDWrite Ypos,Xpos + 24,[Z]
        EndIf
        If BitExternal = 1 Then
            Z = LRead Icon_ExternalControl + X
            LCDWrite Ypos,Xpos + 48,[Z]
        EndIf
        If BitUSB = 1 Then
            Z = LRead Icon_USB + X
            LCDWrite Ypos,Xpos + 72,[Z]
        EndIf
        If BitBacklight = 1 Then
            Z = LRead Icon_Backlight + X
            LCDWrite Ypos,Xpos + 96,[Z]
        EndIf
        
        X = X + 1 
        Next Xpos
    Next Ypos  
    If BitScheduler = 1 Then 
        Print At 4,0,Inverse 0,"SCH:",SCHLine  
    Else
        Print At 4,0,Inverse 0,"                     "
    EndIf
    Print At 5,0,Inverse 0,"LST:",LSTLine
    Print At 6,0,Inverse 0,"ANI:",ANILine
Return

'-----------------------------------------------------------------------
'--- first, paint all icons "empty"                                  ---
'-----------------------------------------------------------------------
ClearIcons:
    BytePos = 0
    For y = 1 To 5
        X = 0
        For Ypos = 1 To 3
            For Xpos = 3 To 26
                Z = LRead Icon_Empty + X
                LCDWrite Ypos,Xpos + BytePos, [Z]
                X = X + 1
            Next Xpos
        Next Ypos
        BytePos = BytePos + 24
    Next y
Return


'-----------------------------------------------------------------------
'--- send configuration bits to main controller                      ---
'-----------------------------------------------------------------------
SendConfigBits:
    '--- backlight ---
    Dout[0] = "B"
    If BitBacklight = 1 Then 
        Dout[1] = "A"
    Else
        Dout[1] = "U"
    EndIf
    GoSub SendData
    '--- scheduler ---
    Dout[0] = "S"
    If BitScheduler = 1 Then
        Dout[1] = "A"
        GoSub AddDateTimeToDout 
        SchYear = 0:SchMonth = 0:SchDay = 0:SchHr = 0:SchMin = 0
        SCHLine = "--/--/----  --:--"
    Else
        Dout[1] = "U"
        SchYear = 99:SchMonth = 99:SchDay=99:SchHr=99:SchMin = 99
        SCHLine = "--/--/----  --:--"
    EndIf
    GoSub SendData
    '--- external control ---
    Dout[0] = "E"
    If BitExternal = 1 Then
        Dout[1] = "A"
    Else
        Dout[1] = "U"
    EndIf
    GoSub SendData
    '--- USB ---
    Dout[0] = "U"
    If BitUSB = 1 Then
        Dout[1] = "A"
    Else
        Dout[1] = "U"
    EndIf
    GoSub SendData
Return

'-----------------------------------------------------------------------
'--- receive data                                                    ---
'-----------------------------------------------------------------------
ReceiveData:
    Repeat 
        High DOutPin
    Until DInPin = 0
    
    For xxx = 0 To 15
        For zzz = 0 To 7
            Repeat
            Until DInClk = 1
            If DInPin = 1 Then
                SetBit yyy,zzz 
            Else
                ClearBit yyy,zzz
            EndIf
            Repeat
                Low DOutPin
            Until DInClk = 0
            High DOutPin
        Next zzz
        Din[xxx] = yyy
    Next xxx 
    Low DOutPin
    DelayUS 5
Return

'-----------------------------------------------------------------------
'--- send data                                                       ---
'-----------------------------------------------------------------------
SendData:
    Repeat
        High DOutPin    
    Until DInPin = 1
    Low DOutPin
    
    For xxx = 0 To 15
        For zzz = 0 To 7
            yyy = Dout[xxx]
            DOutPin = GetBit yyy,zzz
            Repeat 
                High DOutClk
            Until DInPin = 0
            Repeat
                Low DOutClk
            Until DInPin = 1
        Next zzz
    Next xxx
    Low DOutPin
    DelayUS 5
Return

'-----------------------------------------------------------------------
'--- do menu                                                         ---
'-----------------------------------------------------------------------
DoMenu:
    MnuStyle = 1:MnuTitle = "HOOFDMENU       ":BtnA=1:BtnB=1:BtnC=0:BtnD=0:BtnStar=1:BtnCross=1
    MnuItem1 = "Animaties    "
    MnuItem2 = "Speellijsten " 
    mnuItem3 = "Taakplanner  " 
    MnuItem4 = "Instellingen " 
    MnuItem5 = "Datum-tijd   " 
    GoSub DrawMenu    
    Select Case MnuSelect
        Case 0
            DelayMS 250
            Cls
            GoSub PaintIconsOnMainScreen
            Return
        Case 1
            '--- animations ---
            MnuStyle = 2:MnuTitle = "ANIMATIES       ":BtnA=1:BtnB=1:BtnC=0:BtnD=0:BtnStar=1:BtnCross=1:Btn0=1
            MnuType = 1
            GoSub DrawMenu    
        Case 2
            '--- playlists ---
            MnuStyle = 2:MnuTitle = "SPEELLIJSTEN    ":BtnA=1:BtnB=1:BtnC=0:BtnD=0:BtnStar=1:BtnCross=1:Btn0=1
            MnuType = 2
            GoSub DrawMenu    
        Case 3
            '--- scheduler ---
            MnuStyle = 6:MnuTitle = "TAAKPLANNER     ":BtnA=1:BtnB=1:BtnC=0:BtnD=0:BtnStar=0:BtnCross=1:Btn0=1            
            MnuType = 6
            GoSub DrawMenu    
        Case 4
            '--- setup ---
            MnuStyle = 3:MnuTitle = "INSTELLINGEN    ":BtnA=1:BtnB=1:BtnC=1:BtnD=1:BtnStar=1:BtnCross=1:Btn0=0
            MnuItem1 = "Taken    "
            MnuItem2 = "USB      "
            mnuItem3 = "Extern   "
            MnuItem4 = "LCD licht" 
            GoSub DrawMenu
        Case 5
            '--- date-time ---
            MnuStyle = 4:MnuTitle = "DATUM-TIJD      ":BtnA=1:BtnB=1:BtnC=1:BtnD=1:BtnStar=1:BtnCross=1:Btn0=0
            GoSub DrawMenu
    End Select
    DelayMS 250
    If BitExit = 1 Then
        BitExit = 0
        Cls
        GoSub PaintIconsOnMainScreen
        Return
    EndIf
GoTo DoMenu

'-----------------------------------------------------------------------
'--- read settings from EEPROM                                       ---
'-----------------------------------------------------------------------
ReadSetupFromEEPRom:
    TmpByte = ERead 1: If TmpByte = 1 Then BitScheduler = 1 :Else:BitScheduler = 0
    BitUSB = 0  '--- always disable USB at startup ! ---
    TmpByte = ERead 2: If TmpByte = 1 Then BitExternal = 1  :Else:BitExternal = 0
    TmpByte = ERead 3: If TmpByte = 1 Then BitBacklight = 1:Else:BitBacklight = 0
Return

'-----------------------------------------------------------------------
'--- write settings to EEPROM                                        ---
'-----------------------------------------------------------------------
WriteSetupToEEPROM:
    If BitScheduler = 1 Then TmpByte = 1:Else:TmpByte = 0
    EWrite 1,[TmpByte]
    If BitExternal = 1 Then TmpByte = 1:Else:TmpByte = 0
    EWrite 2,[TmpByte]
    If BitBacklight = 1 Then TmpByte = 1:Else:TmpByte = 0
    EWrite 3,[TmpByte]
Return

'-----------------------------------------------------------------------
'--- outlines the menu                                               ---
'-----------------------------------------------------------------------
DrawMenu:
    Cls
    BitExit = 0
    MnuSelect = 1:MnuInv1=1:MnuInv2=0:MnuInv3=0:MnuInv4=0:MnuInv5=0
    Print At 0,0,Inverse 1,MnuTitle
    If BtnA = 1 Then     Print At 0,15,Inverse 1,"A:",$A,"   " :Else: Print At 0,15,Inverse 1,"      "
    If BtnB = 1 Then     Print At 1,15,Inverse 1,"B:",$B,"   " :Else: Print At 1,15,Inverse 1,"      "
    If BtnC = 1 Then     Print At 2,15,Inverse 1,"C:",$C,"   " :Else: Print At 2,15,Inverse 1,"      "
    If BtnD = 1 Then     Print At 3,15,Inverse 1,"D:",$D,"   " :Else: Print At 3,15,Inverse 1,"      "
    If Btn0 = 1 Then     Print At 4,15,Inverse 1,"0:info"      :Else: Print At 4,15,Inverse 1,"      "
    If BtnStar = 1 Then  Print At 6,15,Inverse 1,"*:ok  "      :Else: Print At 6,15,Inverse 1,"      "
    If BtnCross = 1 Then Print At 7,15,Inverse 1,"#:stop"      :Else: Print At 7,15,Inverse 1,"      "
    '--- "clear" lines between blocks on screen ---
    GoSub ClearLinesBetweenButtons
    '--- depending on menu-style, draw other boxes ---
    Select Case MnuStyle
        Case 1
            '----------------------------------------
            '--- simpel menu-structure            ---
            '----------------------------------------
            GoSub DrawBoxFor5Lines:
            UpdateMenuDraw1:
            Print At 2,1,Inverse MnuInv1,MnuItem1
            Print At 3,1,Inverse MnuInv2,MnuItem2
            Print At 4,1,Inverse MnuInv3,mnuItem3
            Print At 5,1,Inverse MnuInv4,MnuItem4
            Print At 6,1,Inverse MnuInv5,MnuItem5
            DelayMS 250
            GoSub WaitForCorrectKey:
            If BtnA=1 Then MnuSelect = MnuSelect - 1
            If BtnB=1 Then MnuSelect = MnuSelect + 1
            If MnuSelect < 1 Then MnuSelect = 5
            If MnuSelect > 5 Then MnuSelect = 1
            GoSub UpdateMenuInv
            If BtnCross = 1 Then 
                MnuSelect = 0
                Return
            EndIf
            If BtnStar = 1 Then Return
            GoTo UpdateMenuDraw1:
        Case 2
            '----------------------------------------
            '--- menu with 5 lines and info-btn   ---
            '----------------------------------------
            GoSub DrawBoxFor5Lines
            BytePos = 0
            'mnutype = 1 : animations
            'mnutype = 2 : playlists
            Select Case MnuType
                Case 1
                    Dout[0] = "A"   
                Case 2
                    Dout[0] = "P"
            End Select
            Dout[1]= "L"
            Dout[2] = 0
            GoSub SetHourGlass
            GoSub SendData
            GoSub ReceiveFileNames
            UpdateMenuDraw2:
            MnuMaxPos = 0
            If MnuItem1[8] = "." Then
                Print At 2,1,Inverse MnuInv1,MnuItem1
                MnuMaxPos = 1
            Else
                Print At 2,1,Inverse 0,"             "
            EndIf
            If MnuItem2[8] = "." Then
                Print At 3,1,Inverse MnuInv2,MnuItem2
                MnuMaxPos = 2
            Else
                Print At 3,1,Inverse 0,"             "
            EndIf
            If mnuItem3[8] = "." Then
                Print At 4,1,Inverse MnuInv3,mnuItem3
                MnuMaxPos = 3
            Else
                Print At 4,1,Inverse 0,"             "
            EndIf
            If MnuItem4[8] = "." Then
                Print At 5,1,Inverse MnuInv4,MnuItem4            
                MnuMaxPos = 4
            Else
                Print At 5,1,Inverse 0,"             "
            EndIf
            If MnuItem5[8] = "." Then
                Print At 6,1,Inverse MnuInv5,MnuItem5
                MnuMaxPos = 5
            Else
                Print At 6,1,Inverse 0,"             "
            EndIf
            DelayMS 200
            If MnuMaxPos = 0 Then
                If BytePos > 0 Then
                    BytePos = BytePos - 1
                    Dout[2] = BytePos
                    GoSub SetHourGlass
                    GoSub SendData
                    GoSub ReceiveFileNames
                    MnuSelect=5
                    GoSub UpdateMenuInv
                    GoTo UpdateMenuDraw2
                Else
                    Print At 3,1,Inverse 0,"  BESTANDEN  "
                    Print At 4,1,Inverse 0,"NIET GEVONDEN"
                    GoSub WaitForCorrectKey
                    Return
                EndIf
            Else
                GoSub WaitForCorrectKey
            EndIf
            If BtnA=1 Then MnuSelect = MnuSelect - 1
            If BtnB=1 Then MnuSelect = MnuSelect + 1
            '--- exit with # ---
            If BtnCross = 1 Then
                Return
            EndIf
            '--- file is selected, so play it !               ---
            '--- (menu = animation or playlist,               ---
            '--- otherwise don't do anything with this button ---
            If BtnStar = 1 Then
                Dout[1]="P"
                Dout[2]=BytePos
                Dout[3]=MnuSelect
                GoSub SendData
                BitExit = 1
                Return
            EndIf
            '--- file info requested ---
            If Btn0=1 Then 
                GoSub SetHourGlass
                GoSub GetFileInfo
                GoSub ClearButtons
                Repeat
                    GoSub WaitForCorrectKey
                Until BtnCross = 1
                GoSub SetHourGlass
                GoSub SetButtons
                Dout[2] = BytePos
                Dout[1] = "L"
                GoSub SendData
                GoSub ReceiveFileNames
                GoTo UpdateMenuDraw2
            EndIf
            If MnuMaxPos = 5 Then                                   
                If MnuSelect > 5 Then 
                    If BytePos < 255 Then
                        MnuSelect = 1
                        BytePos = BytePos + 1
                        Dout[2]=BytePos
                        GoSub SetHourGlass
                        GoSub SendData
                        GoSub ReceiveFileNames
                    Else
                        MnuSelect = 1
                    EndIf
                EndIf
            Else
                If MnuSelect > MnuMaxPos Then
                    MnuSelect = MnuMaxPos
                EndIf
            EndIf
            If MnuSelect < 1 Then 
                If BytePos > 0 Then
                    BytePos = BytePos - 1
                    Dout[2] = BytePos
                    MnuSelect=5
                Else
                    BytePos = 0
                    Dout[2]=BytePos
                    MnuSelect = 1
                EndIf
                GoSub SetHourGlass
                GoSub SendData
                GoSub ReceiveFileNames
            EndIf
            
            GoSub UpdateMenuInv
            GoTo UpdateMenuDraw2:                        
        Case 3
            '----------------------------------------
            '--- menu with 4 on/off items         ---
            '----------------------------------------
            GoSub DrawBoxFor4Lines
            UpdateMenuDraw3:
            Print At 2,1,Inverse 0,MnuItem1
            Print At 3,1,Inverse 0,MnuItem2
            Print At 4,1,Inverse 0,mnuItem3
            Print At 5,1,Inverse 0,MnuItem4
            If BitScheduler = 1 Then Print At 2,11,Inverse MnuInv1,"AAN":Else:Print At 2,11,Inverse MnuInv1,"UIT" 
            If BitUSB       = 1 Then Print At 3,11,Inverse MnuInv2,"AAN":Else:Print At 3,11,Inverse MnuInv2,"UIT" 
            If BitExternal  = 1 Then Print At 4,11,Inverse MnuInv3,"AAN":Else:Print At 4,11,Inverse MnuInv3,"UIT" 
            If BitBacklight = 1 Then Print At 5,11,Inverse MnuInv4,"AAN":Else:Print At 5,11,Inverse MnuInv4,"UIT" 
            DelayMS 250
            GoSub WaitForCorrectKey
            If BtnA=1 Then MnuSelect = MnuSelect - 1
            If BtnB=1 Then MnuSelect = MnuSelect + 1
            If MnuSelect < 1 Then MnuSelect = 4
            If MnuSelect > 4 Then MnuSelect = 1
            GoSub UpdateMenuInv
            If BtnC = 1 Or BtnD = 1 Then
                Select Case MnuSelect
                  Case 1
                    BitScheduler = 1 - BitScheduler
                  Case 2
                    BitUSB = 1 - BitUSB
                  Case 3
                    BitExternal = 1 - BitExternal
                  Case 4
                    BitBacklight = 1 - BitBacklight
                End Select
           EndIf
           If BtnCross = 1 Then
             GoSub ReadSetupFromEEPRom
             MnuSelect = 0
             Return
           EndIf
          If BtnStar = 1 Then
            GoSub WriteSetupToEEPROM
            GoSub SendConfigBits
            If BitUSB = 1 Then 
              DelayMS 250
              GoSub DoUSB
              DelayMS 250
              BitUSB = 0
              GoSub SendConfigBits
              Return
            EndIf
            Return
          EndIf         
          GoTo UpdateMenuDraw3                       
            
        Case 4
            '----------------------------------------
            '--- special case, date/time settings ---
            '----------------------------------------
            GoSub DrawBoxFor5Lines
            Print At 2,1,Inverse 0,"Tijd"
            Print At 5,1,Inverse 0,"Datum"
            GoSub GetTime
            PosVal[1]=0:PosVal[2]=0:PosVal[3]=0:PosVal[4]=0:PosVal[5]=0:PosVal[6]=0
            PosVal[7]=4:PosVal[8]=0:PosVal[9]=1:PosVal[10]=0:PosVal[11]=1:PosVal[12]=0:PosVal[13]=9
            Print At 3,1,Inverse 0,"00:00:00"
            Print At 6,4,Inverse 0,"01/01/2009"
            BitInverse = 0
            GoSub PrintDayName                        
            Xpos=1:Ypos=3:X=1
            Print At 3,1,Inverse 1,PosVal[1]
            UpdateMenuDraw4:
            DelayMS 250
            GoSub WaitForCorrectKey
            '--- numerische waarden ---
            If X!=7 Then
                If Btn0=1 Then PosVal[X]=0:X= X + 1
                If Btn1=1 Then PosVal[X]=1:X= X + 1
                If Btn2=1 Then PosVal[X]=2:X= X + 1
                If Btn3=1 Then PosVal[X]=3:X= X + 1
                If Btn4=1 Then PosVal[X]=4:X= X + 1
                If Btn5=1 Then PosVal[X]=5:X= X + 1
                If Btn6=1 Then PosVal[X]=6:X= X + 1
                If Btn7=1 Then PosVal[X]=7:X= X + 1
                If Btn8=1 Then PosVal[X]=8:X= X + 1
                If Btn9=1 Then PosVal[X]=9:X= X + 1
            EndIf
            '--- specialleke voor dag-naam ---
            If X=7 Then
                If BtnA=1 Then PosVal[7]=PosVal[7]+1
                If BtnB=1 Then PosVal[7]=PosVal[7]-1
                If PosVal[7]>7 Then PosVal[7]=1
                If PosVal[7]<1 Then PosVal[7]=7
            EndIf
            '--- naar vorige of volgende positie ---
            If BtnC=1 Then X = X - 1
            If BtnD=1 Then X = X + 1
            If X < 1 Then X = 13
            If X > 13 Then X = 1
            '--- xpos en ypos berekenen ---
            BitInverse = 0
            Select Case X
                Case 1
                    Ypos=3:Xpos=1
                Case 2
                    Ypos=3:Xpos=2
                Case 3
                    Ypos=3:Xpos=4
                Case 4
                    Ypos=3:Xpos=5
                Case 5
                    Ypos=3:Xpos=7
                Case 6
                    Ypos=3:Xpos=8
                Case 7
                    BitInverse = 1
                    Ypos=6:Xpos=1
                Case 8
                    Ypos=6:Xpos=4
                Case 9
                    Ypos=6:Xpos=5
                Case 10
                    Ypos=6:Xpos=7
                Case 11
                    Ypos=6:Xpos=8
                Case 12
                    Ypos=6:Xpos=12
                Case 13
                    Ypos=6:Xpos=13
            End Select
            Print At 3,1,Inverse 0,DEC1 PosVal[1],DEC1 PosVal[2],":",DEC1 PosVal[3],DEC1 PosVal[4],":",DEC1 PosVal[5],DEC1 PosVal[6]
            Print At 6,4,Inverse 0,DEC1 PosVal[8],DEC1 PosVal[9],"/",DEC1 PosVal[10],DEC1 PosVal[11],"/20",DEC1 PosVal[12],DEC1 PosVal[13]
            
            '--- zet huidige "cursor" in inverse ---
            If X!=7 Then Print At Ypos,Xpos,Inverse 1,DEC1 PosVal[X]
            GoSub PrintDayName
            If BtnCross = 1 Then Return
            If BtnStar = 1 Then
                '--- update date/time RTC ---
                RtcHr    = (PosVal[1]*16) + PosVal[2]
                RtcMin   = (PosVal[3]*16) + PosVal[4]
                RtcSec   = (PosVal[5]*16) + PosVal[6]
                RtcDate  = PosVal[7]
                RtcDay   = (PosVal[8]*16) + PosVal[9]
                RtcMonth = (PosVal[10]*16) + PosVal[11]
                RtcYear  = (PosVal[12]*16) + PosVal[13]
                GoSub SetTime
                SchYear = 0:SchMonth = 0:SchDay = 0:SchHr = 0:SchMin = 0
                Return
            EndIf
         GoTo UpdateMenuDraw4
        Case 6
            '----------------------------------------
            '--- SCHEDULER FEEDBACK               ---
            '----------------------------------------
            BytePos = 0
            Dout[0] = "C" 
            Dout[1]= "L"
            Dout[2] = 0
            GoSub SetHourGlass
            GoSub AddDateTimeToDout
            GoSub SendData
	        GoSub ReceiveFileNames
	        '--------------
	        UpdateMenuDraw6:
            MnuMaxPos = 0
            If MnuItem1[1] = "-" Then
                Print At 2,0,Inverse 0,"               "
            Else
                Print At 2,0,Inverse MnuInv1,MnuItem1
                MnuMaxPos = 1
            EndIf
            If MnuItem2[1] = "-" Then
                Print At 3,0,Inverse 0,"               "
            Else
                Print At 3,0,Inverse MnuInv2,MnuItem2
                MnuMaxPos = 2
            EndIf
            If mnuItem3[1] = "-" Then
                Print At 4,0,Inverse 0,"               "
            Else
                Print At 4,0,Inverse MnuInv3,mnuItem3
                MnuMaxPos = 3
            EndIf
            If MnuItem4[1] = "-" Then
                Print At 5,0,Inverse 0,"               "
            Else
                Print At 5,0,Inverse MnuInv4,MnuItem4
                MnuMaxPos = 4
            EndIf
            If MnuItem5[1] = "-" Then
                Print At 6,0,Inverse 0,"               "
            Else
                Print At 6,0,Inverse MnuInv5,MnuItem5
                MnuMaxPos = 5
            EndIf
            DelayMS 200
            If MnuMaxPos = 0 Then
                If BytePos > 0 Then
                    BytePos = BytePos - 1
                    Dout[2] = BytePos
                    GoSub SetHourGlass
                    'GoSub AddDateTimeToDout
                    GoSub SendData
                    GoSub ReceiveFileNames
                    MnuSelect=5
                    GoSub UpdateMenuInv
                    GoTo UpdateMenuDraw6
                Else
                    Print At 3,0,Inverse 0,"  GEEN TAKEN  "
                    Print At 4,0,Inverse 0,"  GEVONDEN !  "
                    GoSub WaitForCorrectKey
                    Return
                EndIf
            Else
                GoSub WaitForCorrectKey
            EndIf
            If BtnA=1 Then MnuSelect = MnuSelect - 1
            If BtnB=1 Then MnuSelect = MnuSelect + 1
            '--- exit with # ---
            If BtnCross = 1 Then
                Return
            EndIf
            '--- file info requested ---
            If Btn0=1 Then 
                GoSub SetHourGlass     
                'GoSub AddDateTimeToDout    
                GoSub GetFileInfo   
                GoSub ClearButtons
                Repeat
                    GoSub WaitForCorrectKey
                Until BtnCross = 1
                GoSub SetHourGlass
                GoSub SetButtons
                Dout[2] = BytePos
                Dout[1] = "L"
                'GoSub AddDateTimeToDout
                GoSub SendData
                GoSub ReceiveFileNames
                GoTo UpdateMenuDraw6
            EndIf
            If MnuMaxPos = 5 Then                                   
                If MnuSelect > 5 Then 
                    If BytePos < 255 Then
                        MnuSelect = 1
                        BytePos = BytePos + 1
                        Dout[2]=BytePos
                        GoSub SetHourGlass
                        'GoSub AddDateTimeToDout
                        GoSub SendData
                        GoSub ReceiveFileNames
                    Else
                        MnuSelect = 1
                    EndIf
                EndIf
            Else
                If MnuSelect > MnuMaxPos Then
                    MnuSelect = MnuMaxPos
                EndIf
            EndIf
            If MnuSelect < 1 Then 
                If BytePos > 0 Then
                    BytePos = BytePos - 1
                    Dout[2] = BytePos
                    MnuSelect=5
                Else
                    BytePos = 0
                    Dout[2]=BytePos
                    MnuSelect = 1
                EndIf
                GoSub SetHourGlass
                'GoSub AddDateTimeToDout
                GoSub SendData
                GoSub ReceiveFileNames
            EndIf
            
            GoSub UpdateMenuInv
            GoTo UpdateMenuDraw6:                        
            
            
    End Select    
Return

'-----------------------------------------------------------------------
'--- menu related: when setting date/time, we need to "print" the    ---
'--- first 2 characters of the name of the day                       ---
'-----------------------------------------------------------------------
PrintDayName:
    Select Case PosVal[7]
        Case 1
            Print At 6,1,Inverse BitInverse,"MA"
        Case 2
            Print At 6,1,Inverse BitInverse,"DI"
        Case 3
            Print At 6,1,Inverse BitInverse,"WO"
        Case 4
            Print At 6,1,Inverse BitInverse,"DO"
        Case 5
            Print At 6,1,Inverse BitInverse,"VR"
        Case 6
            Print At 6,1,Inverse BitInverse,"ZA"
        Case 7
            Print At 6,1,Inverse BitInverse,"ZO"
        End Select
Return

'-----------------------------------------------------------------------
'--- menu related: for scheduler data, append date/time in Dout (end)---
'--- We add the hexadecimal value of a date-part, into ONE dOut !    ---
'-----------------------------------------------------------------------
AddDateTimeToDout:
    GoSub GetTime
    Dout[10] = RtcYear
    Dout[11] = RtcMonth
    Dout[12] = RtcDay
    Dout[13] = RtcHr
    Dout[14] = RtcMin
    Dout[15] = RtcSec
Return

'-----------------------------------------------------------------------
'--- menu related: get info for selected file/item                   ---
'-----------------------------------------------------------------------
GetFileInfo:
    Dout[1]="I"
    Dout[2]=BytePos
    Dout[3]=MnuSelect
    GoSub SendData
    Repeat 
    Until DInPin = 0
    GoSub ReceiveData
    For zzz = 0 To 15
        Print At 0,zzz,Inverse 1,Din[zzz]
    Next zzz
    GoSub ReceiveFileNames
    If MnuType = 6 Then
        Z = 0
    Else
        Z = 1
    EndIf    
    Print At 2,Z,Inverse 0,MnuItem1
    Print At 3,Z,Inverse 0,MnuItem2
    Print At 4,Z,Inverse 0,mnuItem3
    Print At 5,Z,Inverse 0,MnuItem4
    Print At 6,Z,Inverse 0,MnuItem5
Return

'-----------------------------------------------------------------------
'--- while waiting, show an hourglass onscreen                       ---
'-----------------------------------------------------------------------
SetHourGlass:
   X=0
   For Ypos = 2 To 6
    For Xpos = 30 To 59
        Z = LRead HourGlass + X
        LCDWrite Ypos,Xpos,[Z]
        X = X + 1
    Next Xpos
   Next Ypos

'-----------------------------------------------------------------------
'--- menu related: clear pixels between buttons                      ---
'-----------------------------------------------------------------------
ClearLinesBetweenButtons:
    Line 0,89,0,89,7
    Line 0,0,7,127,7
    Line 0,89,15,127,15
    Line 0,89,23,127,23
    Line 0,89,31,127,31
    Line 0,89,39,127,39
    Line 0,89,55,127,55
    Line 0,89,63,127,63
Return

'-----------------------------------------------------------------------
'--- menu related: set buttons back, after showing info              ---
'-----------------------------------------------------------------------
SetButtons:
    Print At 0,0,Inverse 1,MnuTitle
    Print At 0,15,Inverse 1,"A:",$A,"   " 
    Print At 1,15,Inverse 1,"B:",$B,"   " 
    Print At 2,15,Inverse 1,"      "
    Print At 3,15,Inverse 1,"      "
    Print At 4,15,Inverse 1,"0:info"
    If MnuType = 6 Then
        Print At 6,15,Inverse 1,"      "          
    Else
        Print At 6,15,Inverse 1,"*:ok  "      
    EndIf
    Print At 7,15,Inverse 1,"#:stop"
    GoSub ClearLinesBetweenButtons
Return

'-----------------------------------------------------------------------
'--- menu related: clear allmost all buttons, when showing info      ---
'-----------------------------------------------------------------------
ClearButtons:
    Print At 0,15,Inverse 1,"      " 
    Print At 1,15,Inverse 1,"      " 
    Print At 2,15,Inverse 1,"      "
    Print At 3,15,Inverse 1,"      "
    Print At 4,15,Inverse 1,"      "
    Print At 6,15,Inverse 1,"      "      
    Print At 7,15,Inverse 1,"#:",$10,"   "
    GoSub ClearLinesBetweenButtons
Return
'-----------------------------------------------------------------------
'--- menu related: retrief filenames                                 ---
'-----------------------------------------------------------------------
ReceiveFileNames:
    If MnuType = 6 Then
        Z = 14
    Else
        Z = 13
    EndIf
    For X = 1 To 5
        Repeat 
        Until DInPin = 1
        GoSub ReceiveData
        Select Case X
            Case 1
                MnuItem1 = Left$(Din,Z)
            Case 2
                MnuItem2 = Left$(Din,Z)
            Case 3
                mnuItem3 = Left$(Din,Z)
            Case 4
                MnuItem4 = Left$(Din,Z)
            Case 5
                MnuItem5 = Left$(Din,Z)
        End Select
    Next X
Return

'-----------------------------------------------------------------------
'--- menu related: draws a box, where 5 lines do fit in              ---
'-----------------------------------------------------------------------
DrawBoxFor5Lines:
    Line 1,4,14,85,14
    Line 1,4,57,85,57
    Line 1,4,14,4,57
    Line 1,85,14,85,57
Return

'-----------------------------------------------------------------------
'--- menu related: draws a box, where 4 lines do fit in              ---
'-----------------------------------------------------------------------
DrawBoxFor4Lines:
    Line 1,4,14,85,14
    Line 1,4,49,85,49
    Line 1,4,14,4,49
    Line 1,85,14,85,49
Return

'-----------------------------------------------------------------------
'--- here is the USB part, once we are in here, we can't go out of it---
'--- a hard-reset is necessary !                                     ---
'-----------------------------------------------------------------------
DoUSB:
   '--- write USB logo to fullscreen ...
   Cls
   X = 0
   Print At 0,0,Inverse 1,"GLYPHIX              "
   Print At 7,0,Inverse 1,"Wauters Kris (C) 2008" 
   For Ypos = 1 To 6
    For Xpos = 15 To 110
        Z = LRead USBLogo + X
        LCDWrite Ypos,Xpos,[Z]
        X = X + 1
    Next Xpos
   Next Ypos
   
   USBLoop:
   GoSub ScanKeyPad
   If BtnCross = 1 Or BtnStar = 1 Then Return
   USBPoll		
   If PP0 = ATTACHED_STATE Then
     Print At 0,10,Inverse 1,"USB: Actief" 
     GoSub DoUSBIn
     GoSub DoUSBOut   
     '--- update screen with USB stats (e.g. "blink" some text when receiving/sending data) ---
   
     '--- Write USB received data, via SPI, to main controller, so that it can be put onto the LED matrix ---
   Else
     Print At 0,10,Inverse 1,"USB:Wachten"    
   EndIf
GoTo USBLoop

'-----------------------------------------------------------------------
'--- menu related, wait for A,B,C,D,# or *                           ---
'-----------------------------------------------------------------------
WaitForCorrectKey:
    GoSub ScanKeyPad
    If BtnA=0 And BtnB=0 And BtnC = 0 And BtnD = 0 And BtnStar=0 And BtnCross=0 And Btn0=0 And Btn1=0 And Btn2=0 And Btn3=0 And Btn4=0 And Btn5=0 And Btn6=0 And Btn7=0 And Btn8=0 And Btn9=0 Then GoTo WaitForCorrectKey
Return

'-----------------------------------------------------------------------
'--- menu related, update inverse or not, for the 5 menu-items       ---
'-----------------------------------------------------------------------
UpdateMenuInv:
    Select Case MnuSelect
        Case 1
            MnuInv1=1:MnuInv2=0:MnuInv3=0:MnuInv4=0:MnuInv5=0
        Case 2
            MnuInv1=0:MnuInv2=1:MnuInv3=0:MnuInv4=0:MnuInv5=0
        Case 3
            MnuInv1=0:MnuInv2=0:MnuInv3=1:MnuInv4=0:MnuInv5=0
        Case 4
            MnuInv1=0:MnuInv2=0:MnuInv3=0:MnuInv4=1:MnuInv5=0
        Case 5
            MnuInv1=0:MnuInv2=0:MnuInv3=0:MnuInv4=0:MnuInv5=1
    End Select 
Return

'-----------------------------------------------------------------------
'--- set Btn-variables accordingly the pressed buttons               ---
'-----------------------------------------------------------------------
ScanKeyPad:
   Low Row4:High Row1
    Btn1 = 0:Btn4 = 0:Btn7 = 0:BtnStar = 0
    If Col1 = 1 Then Btn1 = 1
    If Col2 = 1 Then Btn4 = 1
    If Col3 = 1 Then Btn7 = 1
    If Col4 = 1 Then BtnStar = 1
   Low Row1:High Row2
    Btn2 = 0:Btn5 = 0:Btn8 = 0:Btn0 = 0
    If Col1 = 1 Then Btn2 = 1
    If Col2 = 1 Then Btn5 = 1
    If Col3 = 1 Then Btn8 = 1
    If Col4 = 1 Then Btn0 = 1
   Low Row2:High Row3
    Btn3 = 0:Btn6 = 0:Btn9 = 0:BtnCross = 0
    If Col1 = 1 Then Btn3 = 1
    If Col2 = 1 Then Btn6 = 1
    If Col3 = 1 Then Btn9 = 1
    If Col4 = 1 Then BtnCross = 1
   Low Row3:High Row4
    BtnA = 0:BtnB = 0:BtnC = 0:BtnD = 0
    If Col1 = 1 Then BtnA = 1
    If Col2 = 1 Then BtnB = 1
    If Col3 = 1 Then BtnC = 1
    If Col4 = 1 Then BtnD = 1
Return

'-----------------------------------------------------------------------
'--- DS1302 RTC PART                                                 ---
'-----------------------------------------------------------------------
'***********************************************************************
'* Initialise the RealTimeClock when starting up                       *
'* when clock is not running previously, set it to donderdag 01/01/2009* 
'***********************************************************************
InitClock:
    DelayMS 150
    Low RST                         ' Reset RTC
    Low SCLK
    DelayMS 150

    GoSub GetTime
    TmpSeconds = RtcSec
    DelayMS 1200
    GoSub GetTime
    If TmpSeconds = RtcSec Then 'clock is not running previously
        RtcYear = $09
        RtcDay = $01
        RtcMonth = $01
        RtcDate = $04       'day-number (1=monday ... 7 = sunday)
        RtcHr = $12
        RtcMin = $12
        RtcSec = $12
        GoSub SetTime:
    EndIf
Return 

'***********************************************************************
'* Read Time from RTC                                                  *
'***********************************************************************
GetTime:
        RST = 1         					' Ready for transfer
        'DelayMS 1
        SHOut IO,SCLK,lsbfirst,[$bf]     	' Read all 8 RTC registers in burst mode
        'DelayMS 1
        SHIn IO,SCLK,lsbpre,[RtcSec,RtcMin,RtcHr,RtcDay,RtcMonth,RtcDate,RtcYear,RtcControl]
        'DelayMS 1
        RST = 0         					' Reset RTC
        'DelayMS 1
Return     

'***********************************************************************
'* write time to RTC                                                   *
'***********************************************************************
SetTime:
        RST = 1         					' Ready for transfer
        'DelayMS 1
        SHOut IO,SCLK,lsbfirst,[$8e,0]		' Enable write
        'DelayMS 1
        RST = 0         					' Reset RTC
        'DelayMS 1
        RST = 1         					' Ready for transfer
        'DelayMS 1
' Write all 8 RTC registers in burst mode
        SHOut IO,SCLK,lsbfirst,[$be,RtcSec,RtcMin,RtcHr,RtcDay,RtcMonth,RtcDate,RtcYear,0]
        'DelayMS 1
        RST = 0         					' Reset RTC
        'DelayMS 1
        RST = 1
        'DelayMS 1
        SHOut IO,SCLK,lsbfirst,[$90,%10100101]
        'DelayMS 1
        RST = 0
        'DelayMS 1
Return
     
'-----------------------------------------------------------------------
'--- USB PART                                                        ---
'-----------------------------------------------------------------------
' ************************************************************
' * remember, you must keep the USB                          *
' * connection alive with a call to USBPoll, USBIn or USBOut *
' * every couple of milliseconds or so                       *
' ************************************************************
  
' ************************************************************
' * receive data from the USB bus                            *
' ************************************************************
DoUSBIn:
   USBIn 1, USBBuffer, USBBufferSizeRX, DoUSBIn
   Return
   
' ************************************************************
' * transmit data                                            *
' ************************************************************
DoUSBOut:   
   USBOut 1, USBBuffer, USBBufferSizeTX, DoUSBOut
   Return

' ************************************************************
' * wait for USB interface to attach                         *
' ************************************************************
AttachToUSB:
   Repeat								
      USBPoll		
   Until PP0 = ATTACHED_STATE
   Return	


'-----------------------------------------------------------------------
'--- empty Icon                                                      ---
'-----------------------------------------------------------------------
Icon_Empty:
LData As Byte $00,$FE,$02,$02,$02,$02,$02,$02,$02,$02,$02,$02,$02,$02,$02,$02,$02,$02,$02,$02,$02,$02,$FE,$00
LData As Byte $00,$FF,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$FF,$00
LData As Byte $00,$7F,$40,$40,$40,$40,$40,$40,$40,$40,$40,$40,$40,$40,$40,$40,$40,$40,$40,$40,$40,$40,$7F,$00

'-----------------------------------------------------------------------
'--- Icon for mainscreen: external control                           ---
'-----------------------------------------------------------------------
Icon_ExternalControl:
LData As Byte $00,$FE,$02,$02,$E2,$12,$12,$12,$E2,$02,$F2,$02,$E2,$12,$12,$12,$E2,$02,$F2,$02,$02,$02,$FE,$00
LData As Byte $00,$FF,$00,$00,$E3,$04,$C4,$24,$23,$20,$C7,$00,$E3,$04,$C4,$24,$23,$20,$C7,$00,$00,$00,$FF,$00
LData As Byte $00,$7F,$40,$40,$4F,$40,$47,$48,$48,$48,$47,$40,$4F,$40,$47,$48,$48,$48,$47,$40,$40,$40,$7F,$00

'-----------------------------------------------------------------------
'--- Icon for mainscreen: scheduler                                  ---
'-----------------------------------------------------------------------
Icon_Scheduler:
LData As Byte $00,$FE,$02,$02,$82,$42,$22,$12,$12,$12,$0A,$EA,$EA,$0A,$12,$12,$12,$22,$42,$82,$02,$02,$FE,$00
LData As Byte $00,$FF,$00,$3C,$C3,$00,$00,$00,$00,$00,$00,$1F,$1F,$18,$18,$18,$00,$00,$00,$C3,$3C,$00,$FF,$00
LData As Byte $00,$7F,$40,$40,$41,$42,$44,$48,$48,$48,$50,$50,$50,$50,$48,$48,$48,$44,$42,$41,$40,$40,$7F,$00

'-----------------------------------------------------------------------
'--- Icon for mainscreen: backlight                                  ---
'-----------------------------------------------------------------------
Icon_Backlight:
LData As Byte $00,$FE,$02,$02,$02,$02,$42,$82,$02,$02,$02,$BA,$BA,$02,$02,$02,$82,$42,$02,$02,$02,$02,$FE,$00
LData As Byte $00,$FF,$00,$18,$18,$18,$00,$18,$7E,$FF,$FF,$FF,$FF,$FF,$FF,$7E,$18,$00,$18,$18,$18,$00,$FF,$00
LData As Byte $00,$7F,$40,$40,$40,$40,$42,$41,$40,$40,$40,$5D,$5D,$40,$40,$40,$41,$42,$40,$40,$40,$40,$7F,$00

'-----------------------------------------------------------------------
'--- Icon for mainscreen: usb                                        ---
'-----------------------------------------------------------------------
Icon_USB:
LData As Byte $00,$FE,$02,$02,$02,$02,$02,$02,$02,$02,$02,$02,$82,$C2,$C2,$82,$02,$02,$02,$02,$02,$02,$FE,$00
LData As Byte $00,$FF,$00,$1C,$1C,$1C,$08,$0E,$0A,$7B,$C9,$89,$89,$0B,$0B,$89,$88,$88,$3E,$1C,$08,$00,$FF,$00
LData As Byte $00,$7F,$40,$40,$40,$40,$40,$40,$40,$40,$40,$40,$41,$41,$41,$43,$43,$43,$40,$40,$40,$40,$7F,$00

'-----------------------------------------------------------------------
'--- Icon for mainscreen: PlayList                                   ---
'-----------------------------------------------------------------------
Icon_LST:
LData As Byte $00,$FE,$02,$FA,$0A,$4A,$4A,$4A,$4A,$4A,$4A,$0A,$FA,$02,$02,$FA,$82,$82,$82,$82,$02,$02,$FE,$00
LData As Byte $00,$FF,$00,$FF,$00,$55,$55,$55,$55,$55,$55,$00,$FF,$00,$00,$84,$AA,$AA,$AA,$90,$00,$00,$FF,$00
LData As Byte $00,$7F,$40,$5F,$50,$51,$51,$51,$51,$51,$51,$50,$5F,$40,$40,$40,$40,$4F,$40,$40,$40,$40,$7F,$00

'-----------------------------------------------------------------------
'--- Icon for mainscreen: Animation                                  ---
'-----------------------------------------------------------------------
Icon_ANI:
LData As Byte $00,$FE,$02,$FA,$0A,$4A,$4A,$4A,$4A,$4A,$4A,$0A,$FA,$02,$02,$F2,$2A,$2A,$2A,$F2,$02,$02,$FE,$00
LData As Byte $00,$FF,$00,$FF,$00,$55,$55,$55,$55,$55,$55,$00,$FF,$00,$00,$3E,$04,$88,$10,$3E,$00,$00,$FF,$00
LData As Byte $00,$7F,$40,$5F,$50,$51,$51,$51,$51,$51,$51,$50,$5F,$40,$40,$40,$40,$4F,$40,$40,$40,$40,$7F,$00

'-----------------------------------------------------------------------
'--- HourGlass                                                       ---
'-----------------------------------------------------------------------
HourGlass:
LData As Byte $FF,$01,$01,$01,$01,$71,$F1,$F1,$F1,$F1,$F1,$F1,$F1,$F1,$F1,$F1,$F1,$F1,$F1,$F1,$F1,$F1,$F1,$F1,$71,$01,$01,$01,$01,$FF
LData As Byte $FF,$00,$00,$00,$00,$00,$3F,$40,$C0,$80,$20,$40,$A0,$40,$A0,$A0,$40,$A0,$40,$20,$80,$C0,$40,$3F,$00,$00,$00,$00,$00,$FF
LData As Byte $FF,$00,$00,$00,$00,$00,$00,$00,$00,$81,$62,$24,$3C,$03,$51,$51,$03,$3C,$24,$62,$81,$00,$00,$00,$00,$00,$00,$00,$00,$FF
LData As Byte $FF,$00,$00,$00,$00,$00,$FC,$02,$03,$01,$80,$C0,$40,$F0,$5A,$5A,$F0,$40,$C0,$80,$01,$03,$02,$FC,$00,$00,$00,$00,$00,$FF
LData As Byte $FF,$80,$80,$80,$80,$8E,$8F,$8F,$8F,$8F,$8F,$8F,$8F,$8F,$8F,$8F,$8F,$8F,$8F,$8F,$8F,$8F,$8F,$8F,$8E,$80,$80,$80,$80,$FF
'-----------------------------------------------------------------------
'--- USBLogo                                                         ---
'-----------------------------------------------------------------------
USBLogo:
LData As Byte $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
LData As Byte $00,$80,$C0,$E0,$E0,$F0,$F0,$F0,$F0,$E0,$E0,$C0,$80,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
LData As Byte $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$80,$E0,$F0,$F0,$F8,$78,$7C,$3C,$3C,$3C,$1C,$1C,$1E,$1E,$1E
LData As Byte $3E,$7F,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$3F,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
LData As Byte $00,$00,$00,$00,$00,$C0,$F0,$F8,$F8,$FC,$FC,$FC,$FC,$FC,$FC,$F8,$F0,$E0,$C0,$C0,$C0,$C0,$C0,$C0,$C0,$C0,$C0,$C0,$C0,$C0,$C0,$F8,$FE,$FF,$FF,$CF,$C3,$C0,$C0,$C0,$C0,$C0,$C0,$C0,$C0,$C0,$C0,$C0
LData As Byte $C0,$C0,$C0,$C1,$C3,$C3,$C3,$C3,$C3,$C3,$C1,$C1,$C0,$C0,$C0,$C0,$C0,$C0,$C0,$C0,$C0,$C0,$C0,$C0,$C0,$C0,$C0,$C0,$C0,$C0,$FE,$FC,$FC,$F8,$F0,$F0,$E0,$E0,$C0,$C0,$80,$80,$00,$00,$00,$00,$00,$00
LData As Byte $00,$00,$00,$00,$00,$0F,$1F,$3F,$7F,$7F,$FF,$FF,$FF,$FF,$7F,$7F,$3F,$1F,$07,$07,$07,$07,$07,$07,$07,$07,$07,$07,$07,$07,$07,$07,$07,$07,$07,$07,$07,$07,$0F,$1F,$7F,$FF,$FF,$E7,$C7,$07,$07,$07
LData As Byte $07,$07,$07,$07,$07,$07,$07,$07,$07,$07,$07,$07,$07,$07,$07,$07,$07,$07,$07,$07,$07,$07,$07,$07,$07,$07,$07,$07,$07,$07,$FF,$7F,$7F,$3F,$3F,$1F,$1F,$0F,$07,$07,$03,$03,$01,$00,$00,$00,$00,$00
LData As Byte $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$01,$03,$0F,$0F,$1F,$3E,$3C
LData As Byte $7C,$78,$78,$F8,$F0,$F0,$F0,$F0,$F0,$F0,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
LData As Byte $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
LData As Byte $00,$00,$00,$00,$00,$00,$00,$00,$00,$01,$0F,$0F,$0F,$0F,$0F,$0F,$0F,$0F,$0F,$0F,$0F,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00

'-----------------------------------------------------------------------
'--- startup screen                                                  ---
'-----------------------------------------------------------------------
StartupScreen:
LData As Byte $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
LData As Byte $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
LData As Byte $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
LData As Byte $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$F0,$F0,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
LData As Byte $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$30,$F0,$C0,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$80,$C0,$60,$30,$18,$08,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
LData As Byte $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$80,$C0,$E0,$60,$30,$18,$08,$0C,$06,$06,$02,$02,$03,$01,$01,$03,$03,$02,$00,$00,$00,$00,$00
LData As Byte $F0,$FF,$0F,$00,$00,$00,$00,$00,$00,$00,$00,$00,$FC,$FC,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$FC,$FC,$F8,$00,$00,$10,$30,$38,$18,$FC,$FC,$88,$18,$18,$30,$30,$E0,$C0,$00,$00,$00,$00,$00,$00
LData As Byte $FF,$FF,$00,$00,$80,$E0,$70,$38,$18,$F8,$F0,$00,$00,$00,$00,$00,$C0,$F8,$F8,$00,$00,$00,$00,$00,$00,$00,$00,$00,$0C,$3C,$F8,$C0,$C0,$E0,$70,$18,$0C,$04,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
LData As Byte $00,$00,$00,$00,$00,$00,$00,$00,$1E,$3F,$71,$60,$40,$40,$40,$40,$40,$40,$60,$20,$20,$20,$30,$10,$10,$FE,$FE,$1E,$02,$00,$00,$00,$7F,$FF,$80,$C0,$C0,$60,$30,$10,$00,$00,$00,$00,$FF,$FF,$80,$80
LData As Byte $C0,$E0,$70,$30,$18,$0C,$06,$03,$01,$FF,$FF,$00,$00,$00,$00,$00,$00,$FF,$FF,$FF,$40,$40,$40,$40,$40,$73,$3F,$1C,$00,$00,$00,$78,$FF,$FF,$7C,$1F,$07,$00,$00,$00,$00,$FF,$FF,$80,$00,$00,$00,$00
LData As Byte $FF,$FF,$80,$C0,$60,$60,$30,$10,$00,$00,$C0,$E0,$38,$1C,$07,$03,$07,$1C,$38,$60,$C0,$80,$80,$80,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
LData As Byte $00,$00,$00,$00,$00,$00,$00,$00,$40,$7F,$7F,$20,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$E0,$F0,$19,$09,$09,$00,$00,$00,$00,$00,$00,$00,$00,$80,$FF,$7F,$00,$00,$00,$00,$00
LData As Byte $00,$FF,$FF,$01,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$01,$01,$00,$00,$00,$00,$00,$00,$00,$00,$01,$01,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$06,$07,$01,$00,$00,$00,$00,$00
LData As Byte $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
LData As Byte $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$01,$03,$06,$0C,$08,$08,$18,$10,$10,$18,$08,$0C,$06,$07,$01,$00,$00,$00,$00,$00,$00,$00,$07,$07,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
LData As Byte $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
LData As Byte $00,$00,$00,$70,$F0,$80,$00,$F0,$F0,$00,$80,$F0,$70,$00,$00,$40,$40,$40,$C0,$80,$00,$C0,$C0,$00,$00,$C0,$C0,$00,$F0,$F0,$40,$40,$00,$80,$C0,$40,$40,$C0,$80,$00,$C0,$C0,$80,$C0,$00,$80,$C0,$40
LData As Byte $40,$40,$00,$00,$00,$00,$F0,$F0,$C0,$60,$30,$10,$00,$C0,$C0,$80,$C0,$00,$D0,$D0,$00,$80,$C0,$40,$40,$40,$00,$00,$00,$00,$C0,$F0,$30,$00,$00,$E0,$F0,$10,$10,$10,$10,$00,$00,$30,$F0,$C0,$00,$00
LData As Byte $00,$00,$20,$30,$10,$90,$F0,$60,$00,$E0,$F0,$10,$10,$F0,$E0,$00,$E0,$F0,$10,$10,$F0,$E0,$00,$60,$F0,$90,$90,$F0,$60,$00,$00,$00,$00,$00,$00,$00,$03,$0F,$0F,$01,$01,$0F,$0F,$03,$00,$00,$06,$0F
LData As Byte $09,$09,$0F,$0F,$00,$07,$0F,$08,$08,$0F,$0F,$00,$07,$0F,$08,$08,$00,$07,$0F,$09,$09,$09,$09,$00,$0F,$0F,$00,$00,$00,$09,$0B,$0B,$0F,$06,$00,$00,$00,$00,$0F,$0F,$03,$06,$0C,$08,$00,$0F,$0F,$00
LData As Byte $00,$00,$0F,$0F,$00,$09,$0B,$0B,$0F,$06,$00,$00,$00,$00,$07,$1F,$38,$20,$00,$07,$0F,$08,$08,$08,$08,$00,$20,$38,$1F,$07,$00,$00,$00,$00,$0C,$0E,$0B,$09,$08,$08,$00,$07,$0F,$08,$08,$0F,$07,$00
LData As Byte $07,$0F,$08,$08,$0F,$07,$00,$07,$0F,$08,$08,$0F,$07,$00,$00,$00
  
'-----------------------------------------------------------------------
'--- INTERNAL FONT DECLARATION                                       ---
'-----------------------------------------------------------------------
Font:-	CData $3E,$51,$49,$45,$3E,$00		'48 - 0 - 30
		CData $00,$42,$7F,$40,$00,$00		'49 - 1 - 31
		CData $42,$61,$51,$49,$46,$00		'50 - 2 - 32
		CData $21,$41,$45,$4B,$31,$00		'51 - 3 - 33
		CData $18,$14,$12,$7F,$10,$00		'52 - 4 - 34
		CData $27,$45,$45,$45,$39,$00		'53 - 5 - 35
		CData $3C,$4A,$49,$49,$30,$00		'54 - 6 - 36
		CData $01,$71,$09,$05,$03,$00		'55 - 7 - 37
		CData $36,$49,$49,$49,$36,$00		'56 - 8 - 38
		CData $06,$49,$49,$49,$3E,$00		'57 - 9 - 39
        CData $04,$06,$7F,$06,$04,$00		'Graphic character 10  UP    arrow
        CData $10,$30,$7F,$30,$10,$00		'Graphic character 11  DOWN  arrow
        CData $08,$1C,$3E,$08,$08,$00		'Graphic character 12  LEFT  arrow
        CData $08,$08,$3E,$1C,$08,$00		'Graphic character 13  RIGHT arrow
        CData $3E,$41,$5D,$55,$41,$3E		'Graphic character 14  (C) copyright
        CData $3E,$41,$4D,$49,$41,$3E		'Graphic character 15  small clock
        CData $12,$39,$7D,$11,$11,$0E		'Graphic character 16  "return" arrow
        CData $AA,$55,$AA,$55,$AA,$55		'Graphic character 17
        CData $01,$01,$01,$01,$01,$01		'Graphic character 18
        CData $80,$80,$80,$80,$80,$80		'Graphic character 19
        CData $FF,$00,$00,$00,$00,$00		'Graphic character 20
        CData $00,$00,$00,$00,$00,$FF		'Graphic character 21
        CData $FF,$01,$01,$01,$01,$01		'Graphic character 22
        CData $01,$01,$01,$01,$01,$FF		'Graphic character 23
        CData $FF,$80,$80,$80,$80,$80		'Graphic character 24
        CData $80,$80,$80,$80,$80,$FF		'Graphic character 25
        CData $00,$00,$00,$00,$F0,$F0		'User defined character 26
        CData $00,$00,$00,$00,$0F,$0F		'User defined character 27
        CData $00,$00,$00,$00,$00,$00		'User defined character 28
        CData $00,$00,$00,$00,$00,$00		'User defined character 29
        CData $00,$00,$00,$00,$00,$00		'User defined character 30
        CData $00,$00,$00,$00,$00,$00		'User defined character 31
        CData $00,$00,$00,$00,$00,$00		'32 -   - 20        
		CData $00,$00,$4F,$00,$00,$00		'33 - ! - 21
		CData $00,$07,$00,$07,$00,$00		'34 - " - 22
		CData $14,$7F,$14,$7F,$14,$00		'35 - # - 23
		CData $24,$2A,$7F,$2A,$12,$00		'36 - $ - 24
		CData $23,$13,$08,$64,$62,$00		'37 - % - 25
		CData $36,$49,$55,$22,$50,$00		'38 - & - 26
		CData $00,$05,$03,$00,$00,$00		'39 - ' - 27
		CData $1C,$22,$41,$00,$00,$00		'40 - ( - 28
		CData $00,$00,$41,$22,$1C,$00		'41 - ) - 29
		CData $14,$08,$3E,$08,$14,$00		'42 - * - 2A
		CData $08,$08,$3E,$08,$08,$00		'43 - + - 2B
		CData $00,$50,$30,$00,$00,$00		'44 - , - 2C
		CData $08,$08,$08,$08,$08,$00		'45 - - - 2D
		CData $00,$60,$60,$00,$00,$00		'46 - . - 2E
		CData $20,$10,$08,$04,$02,$00		'47 - / - 2F
		CData $3E,$51,$49,$45,$3E,$00		'48 - 0 - 30
		CData $00,$42,$7F,$40,$00,$00		'49 - 1 - 31
		CData $42,$61,$51,$49,$46,$00		'50 - 2 - 32
		CData $21,$41,$45,$4B,$31,$00		'51 - 3 - 33
		CData $18,$14,$12,$7F,$10,$00		'52 - 4 - 34
		CData $27,$45,$45,$45,$39,$00		'53 - 5 - 35
		CData $3C,$4A,$49,$49,$30,$00		'54 - 6 - 36
		CData $01,$71,$09,$05,$03,$00		'55 - 7 - 37
		CData $36,$49,$49,$49,$36,$00		'56 - 8 - 38
		CData $06,$49,$49,$49,$3E,$00		'57 - 9 - 39
		CData $00,$36,$36,$00,$00,$00		'58 - : - 3A
		CData $00,$56,$36,$00,$00,$00		'59 - ; - 3B
		CData $08,$14,$22,$41,$00,$00		'60 - < - 3C
		CData $14,$14,$14,$14,$14,$00		'61 - = - 3D
		CData $00,$41,$22,$14,$08,$00		'62 - > - 3E
		CData $02,$01,$51,$09,$06,$00		'63 - ? - 3F
		CData $32,$49,$79,$41,$3E,$00		'64 - @ - 40
		CData $7E,$11,$11,$11,$7E,$00		'65 - A - 41
		CData $7F,$49,$49,$49,$36,$00		'66 - B - 42
		CData $3E,$41,$41,$41,$22,$00		'67 - C - 43
		CData $7F,$41,$41,$22,$1C,$00		'68 - D - 44
		CData $7F,$49,$49,$49,$41,$00		'69 - E - 45
		CData $7F,$09,$09,$09,$01,$00		'70 - F - 46
		CData $3E,$41,$49,$49,$7A,$00		'71 - G - 47
		CData $7F,$08,$08,$08,$7F,$00		'72 - H - 48
		CData $00,$41,$7F,$41,$00,$00		'73 - I - 49
		CData $20,$40,$41,$3F,$01,$00		'74 - J - 4A
		CData $7F,$08,$14,$22,$41,$00		'75 - K - 4B
		CData $7F,$40,$40,$40,$40,$00		'76 - L - 4C
		CData $7F,$02,$0C,$02,$7F,$00		'77 - M - 4D
		CData $7F,$04,$08,$10,$7F,$00		'78 - N - 4E
		CData $3E,$41,$41,$41,$3E,$00		'79 - O - 4F
		CData $7F,$09,$09,$09,$06,$00		'80 - P - 50
		CData $3E,$41,$51,$21,$5E,$00		'81 - Q - 51
		CData $7F,$09,$19,$29,$46,$00		'82 - R - 52
		CData $46,$49,$49,$49,$31,$00		'83 - S - 53
		CData $01,$01,$7F,$01,$01,$00		'84 - T - 54
		CData $3F,$40,$40,$40,$3F,$00		'85 - U - 55
		CData $1F,$20,$40,$20,$1F,$00		'86 - V - 56
		CData $3F,$40,$38,$40,$3F,$00		'87 - W - 57
		CData $63,$14,$08,$14,$63,$00		'88 - X - 58
		CData $07,$08,$70,$08,$07,$00		'89 - Y - 59
		CData $61,$51,$49,$45,$43,$00		'90 - Z - 5A
		CData $7F,$41,$41,$00,$00,$00		'91 - [ - 5B
		CData $02,$04,$08,$10,$20,$00		'92 - \ - 5C
		CData $00,$00,$41,$41,$7F,$00		'93 - ] - 5D
		CData $04,$02,$01,$02,$04,$00		'94 - ^ - 5E
		CData $40,$40,$40,$40,$40,$00		'95 - _ - 5F
		CData $00,$01,$02,$04,$00,$00		'96 - ` - 60
		CData $20,$54,$54,$54,$78,$00		'97 - a - 61
		CData $7F,$48,$44,$44,$38,$00		'98 - b - 62
		CData $38,$44,$44,$44,$20,$00		'99 - c - 63
		CData $38,$44,$44,$48,$7F,$00		'100  d - 64
		CData $38,$54,$54,$54,$18,$00		'101  e - 65
		CData $08,$7E,$09,$01,$02,$00		'102  f - 66
		CData $0C,$52,$52,$52,$3E,$00		'103  g - 67
		CData $7F,$08,$04,$04,$78,$00		'104  h - 68
		CData $00,$44,$7D,$40,$00,$00		'105  i - 69
		CData $00,$20,$40,$44,$3D,$00		'106  j - 6A
		CData $7F,$10,$28,$44,$00,$00		'107  k - 6B
		CData $00,$41,$7F,$40,$00,$00		'108  l - 6C
		CData $7C,$04,$18,$04,$78,$00		'109  m - 6D
		CData $7C,$08,$04,$04,$78,$00		'110  n - 6E
		CData $38,$44,$44,$44,$38,$00		'111  o - 6F
		CData $7C,$14,$14,$14,$08,$00		'112  p - 70
		CData $08,$14,$14,$18,$7C,$00		'113  q - 71
		CData $7C,$08,$04,$04,$08,$00		'114  r - 72
		CData $48,$54,$54,$54,$20,$00		'115  s - 73
		CData $04,$3F,$44,$40,$20,$00		'116  t - 74
		CData $3C,$40,$40,$20,$7C,$00		'117  u - 75
		CData $1C,$20,$40,$20,$1C,$00		'118  v - 76
		CData $3C,$40,$30,$40,$3C,$00		'119  w - 77
		CData $44,$28,$10,$28,$44,$00		'120  x - 78
		CData $0C,$50,$50,$50,$3C,$00		'121  y - 79
		CData $44,$64,$54,$4C,$44,$00		'122  z - 7A
		CData $08,$36,$41,$00,$00,$00		'123  { - 7B
		CData $00,$00,$7F,$00,$00,$00		'124  | - 7C
		CData $00,$00,$41,$36,$08,$00		'125  } - 7D
		CData $00,$08,$04,$08,$04,$00 		'126  ~ - 7E
