 LIST

#DEFINE	__EP0_BUFF_SIZE			64				; 8, 16, 32, or 64
#DEFINE	__MAX_NUM_INT			1				; For tracking Alternate Setting
#DEFINE	__MAX_EP_NUMBER			1				; UEP1
#DEFINE	__NUM_CONFIGURATIONS	        1
#DEFINE	__NUM_INTERFACES		1

#DEFINE	__MODE_PP			_PPBM0
#DEFINE	__UCFG_VAL			_PUEN | _TRINT | _FS | __MODE_PP	; Full-speed

#DEFINE __USB_USE_HID		       	; Inform the rest of the library subroutines that we're using a HID interface

; HID
; Endpoints Allocation
#DEFINE	__HID_INTF_ID			0
#DEFINE	__HID_UEP			UEP1
#DEFINE	__HID_BD_OUT			__ep1Bo
#DEFINE	__HID_INT_OUT_EP_SIZE	        64
#DEFINE	__HID_BD_IN			__ep1Bi
#DEFINE	__HID_INT_IN_EP_SIZE	        64
#DEFINE	__HID_NUM_OF_DSC		1

; ------------------------------------------------------------------------------
; This table is polled by the host immediately after USB Reset has been released.
; The table defines the maximum packet size EP0 can take.
; See section 9.6.1 of the Rev 1.0 USB specification.
; These fields are application dependent to be modified to meet your specifications.
;
__DeviceDescriptor
	retlw	(__EndDeviceDescriptor - __DeviceDescriptor) / 2	; bLength Length of this descriptor
	retlw	__DSC_DEV						; bDescType This is a DEVICE descriptor
	dt	0x10, 0x01						; bcdUSB Revision 1.10 low byte, high byte
	retlw	0x00							; bDeviceClass zero means each interface operates independently
	retlw	0x00							; bDeviceSubClass
	retlw	0x00							; bDeviceProtocol
	retlw	__EP0_BUFF_SIZE						; bMaxPacketSize for EP0

        ; idVendor low byte, high byte
	dt	0x81, 0x17
        ; idProduct low byte, high byte
	dt	0xD0, 0x07

	dt	0x00, 0x00						; bcdDevice low byte, high byte
	retlw	0x01							; iManufacturer
	retlw	0x02							; iProduct

        ; iSerialNumber index
	retlw	0x00

	retlw	__NUM_CONFIGURATIONS 				        ; bNumConfigurations
__EndDeviceDescriptor

; ------------------------------------------------------------------------------
; This table is retrieved by the host after the address has been set.
; This table defines the configurations available for the device.
; See section 9.6.2 of the Rev 1.0 USB specification (page 184).
; These fields are application dependent to be modified to meet your specifications.
;
; Configuration pointer table
__USB_CD_Ptr
__Configs
	db	Low ___Config1, High ___Config1
	db	Low ___Config1, High ___Config1

; Configuration Descriptor
___Config1
	retlw	(__Interface1-___Config1) / 2	; bLength Length of this descriptor
	retlw	__DSC_CFG						; bDescType 2 = CONFIGURATION
__Config1Len
	retlw	Low ((__EndConfig1 - ___Config1) / 2)	; Length of this configuration
	retlw	High ((__EndConfig1 - ___Config1) / 2)
	retlw	0x01							; bNumInterfaces Number of interfaces
	retlw	0x01							; bConfigValue Configuration Value
	retlw	0x04							; iConfig String Index for this config = #01
	retlw	0xA0							; bmAttributes attributes - bus powered

        ; Max power consumption (2X mA)
	retlw	0x32
__Interface1
	retlw	(__HIDDescriptor1 - __Interface1) / 2	; length of descriptor
	retlw	__DSC_INTF
	retlw	0x00							; number of interface, 0 based array
	retlw	0x00							; alternate setting
	retlw	0x02							; number of endpoints used in this interface
	retlw	0x03							; interface CLASS - assigned by the USB
	retlw	0x00							; boot device
	retlw	0x00							; interface protocol - mouse
	retlw 	0x00							; index to string descriptor that describes this interface
__HIDDescriptor1
	retlw	(__Endpoint1In - __HIDDescriptor1) / 2	; descriptor size (9 bytes)
  	retlw	0x21							; descriptor type (HID)
	dt		0x00, 0x00						; HID CLASS release number (1.00) low byte, high byte
   	retlw	0x00							; Localised country code (none)
 	retlw	0x01							; Number of HID CLASS descriptor to follow (1)
 	retlw	0x22							; Report descriptor type (HID)
__ReportDescriptor1Len
	retlw	Low ((__EndReportDescriptor1 - __ReportDescriptor1) / 2)
	retlw	High ((__EndReportDescriptor1 - __ReportDescriptor1) / 2)
__Endpoint1In
	retlw	(__EndPoint1Out - __Endpoint1In) / 2	; length of descriptor
	retlw	__DSC_EP
	retlw	0x81							; EP1, IN
	retlw	0x03							; Interrupt
	dt	0x40, 0x00						; This should be the size of the endpoint buffer low byte, high byte
	retlw	0x0A							; Polling interval
__EndPoint1Out
	retlw	(__EndConfig1 - __EndPoint1Out) / 2	; Length of this Endpoint Descriptor
	retlw	__DSC_EP						; bDescriptorType = 5 for Endpoint Descriptor
	retlw	0x01							; Endpoint number & direction
	retlw	0x03							; Transfer type supported by this Endpoint
	dt	0x40, 0x00						; This should be the size of the endpoint buffer low byte, high byte
	retlw	0x0A							; Polling interval 
__EndConfig1

__ReportDescriptor1
    ; vendor defined usage page
    retlw	0x06		
	retlw	0x00       
    retlw	0xFF

    ; vendor defined usage
    retlw	0x09		
    retlw	0x00

    ; collection(application)
    retlw	0xA1		
    retlw	0x01

    ; *** INPUT REPORT ***
               
    ; vendor defined usage
    retlw	0x09		
    retlw	0x01

    retlw	0x15		; logical minimum (-128)
    retlw	0x80        ; 
    retlw	0x25		; logical maximum (127)
    retlw	0x7F        ; 
    retlw	0x35		; Physical Minimum (0)
    retlw	0x00        ;
    retlw	0x45		; Physical Maximum (255)
    retlw	0xFF        ;
        
    ; report size in bits
    retlw	0x75	
	retlw	0x08
        
    ; report count (number of fields)
    retlw	0x95		
	retlw	0x08
        
    ; Input (Data, Variable, Absolute)
    retlw	0x81		
	retlw	0x02

    ; *** OUTPUT REPORT ***
    
    ; vendor defined usage
    retlw	0x09		; usage (Vendor Defined)
	retlw	0x02        ;

	retlw	0x15		; logical minimum (-128)
	retlw	0x80        ; 
    retlw	0x25		; logical maximum (127)
	retlw	0x7F        ; 
    retlw	0x35		; Physical Minimum (0)
	retlw	0x00        ;
    retlw	0x45		; Physical Maximum (255)
	retlw	0xFF        ;
        
    ; report size in bits
    retlw	0x75		
	retlw	0x08
        
    ; report count (number of fields)
    retlw	0x95	
	retlw	0x08
        
    ; Output (Data, Variable, Absolute)
    retlw	0x91		
	retlw	0x02

	retlw   0xC0		; end collection
__EndReportDescriptor1

; String pointer table
__USB_SD_Ptr
__Strings
	db	Low __String0, High __String0
	db	Low __String1, High __String1
	db	Low __String2, High __String2
	db	Low __String3, High __String3
	db	Low __String4, High __String4

__String0
	retlw	(__String1 - __String0) / 2		; Length of string 
	retlw	__DSC_STR						; Descriptor type 3
	dt   0x09, 0x04    						; Language ID (0x0409 as defined by MS)

;company name
__String1
	retlw	(__String2 - __String1) / 2
	retlw	__DSC_STR
	
        retlw   'W'
        retlw   0x00
        retlw   'a'
        retlw   0x00
        retlw   'u'
        retlw   0x00
        retlw   't'
        retlw   0x00
        retlw   'e'
        retlw   0x00
        retlw   'r'
        retlw   0x00
        retlw   's'
        retlw   0x00
        retlw   '-'
        retlw   0x00
        retlw   'P'
        retlw   0x00
        retlw   'a'
        retlw   0x00
        retlw   'u'
        retlw   0x00
        retlw   'w'
        retlw   0x00
        retlw   'e'
        retlw   0x00
        retlw   'l'
        retlw   0x00
        retlw   's'
        retlw   0x00


;product name
__String2
	retlw	(__String3 - __String2) / 2
	retlw	__DSC_STR
	
        retlw   'G'
        retlw   0x00
        retlw   'l'
        retlw   0x00
        retlw   'y'
        retlw   0x00
        retlw   'p'
        retlw   0x00
        retlw   'h'
        retlw   0x00
        retlw   'i'
        retlw   0x00
        retlw   'x'
        retlw   0x00
        retlw   ' '
        retlw   0x00
        retlw   'L'
        retlw   0x00
        retlw   'E'
        retlw   0x00
        retlw   'D'
        retlw   0x00
        retlw   ' '
        retlw   0x00
        retlw   'M'
        retlw   0x00
        retlw   'a'
        retlw   0x00
        retlw   't'
        retlw   0x00
        retlw   'r'
        retlw   0x00
        retlw   'i'
        retlw   0x00
        retlw   'x'
        retlw   0x00


; serial number
__String3
	retlw	(__String4 - __String3) / 2
	retlw	__DSC_STR
	

__String4
	retlw	(__String5 - __String4) / 2
	retlw	__DSC_STR
	dt	0
__String5
 
 
