Attribute VB_Name = "ModMemoryCardHandling"
'--- declares for memorycard info ---
Public Enum PARTITION_TYPE
    PARTITION_ENTRY_UNUSED = &H0
    PARTITION_FAT_12 = &H1
    PARTITION_XENIX_1 = &H2
    PARTITION_XENIX_2 = &H3
    PARTITION_FAT_16 = &H4
    PARTITION_EXTENDED = &H5
    PARTITION_HUGE = &H6
    PARTITION_IFS = &H7
    PARTITION_OS2BOOTMGR = &HA
    PARTITION_FAT32 = &HB
    PARTITION_FAT32_XINT13 = &HC
    PARTITION_XINT13 = &HE
    PARTITION_XINT13_EXTENDED = &HF
    PARTITION_PREP = &H41
    PARTITION_LDM = &H42
    PARTITION_UNIX = &H63
End Enum

Public Type OVERLAPPED
    ternal As Long
    ternalHigh As Long
    Offset As Long
    OffsetHigh As Long
    hEvent As Long
End Type

Public Type PARTITION_INFORMATION
  StartingOffset As Currency
  PartitionLength As Currency
  HiddenSectors As Long
  PartitionNumber As Currency
  PartitionType As PARTITION_TYPE
  BootIndicator As Byte
  RecognizedPartition As Byte
  RewritePartition As Byte
End Type

'--- declares for data-manipulations on memorycard ---
Public Const GENERIC_WRITE = &H40000000
Public Const GENERIC_READ = &H80000000

Public Const FILE_SHARE_READ = &H1
Public Const FILE_SHARE_WRITE = &H2

Public Const FILE_ATTRIBUTE_NORMAL = &H80

Public Const CREATE_ALWAYS = 2
Public Const OPEN_ALWAYS = 4
Public Const OPEN_EXISTING = 3
Public Const TRUNCATE_EXISTING = 5

Public Const INVALID_HANDLE_VALUE = -1

Public Const IOCTL_DISK_GET_DRIVE_GEOMETRY = &H70000
Public Const IOCTL_DISK_GET_PARTITION_INFO = &H74004

'--- declares for API calls ---
Private Declare Function GetLogicalDriveStrings Lib "kernel32" Alias "GetLogicalDriveStringsA" (ByVal nBufferLength As Long, ByVal lpBuffer As String) As Long
Public Declare Function GetDriveTypeAPI Lib "kernel32" Alias "GetDriveTypeA" (ByVal nDrive As String) As Long
Public Declare Function DeviceIoControl Lib "kernel32.dll" (ByVal hDevice As Long, ByVal dwIoControlCode As Long, lpInBuffer As Any, ByVal nInBufferSize As Long, lpOutBuffer As Any, ByVal nOutBufferSize As Long, lpBytesReturned As Long, lpOverlapped As OVERLAPPED) As Long
Public Declare Function CloseHandle Lib "kernel32" (ByVal hObject As Long) As Long
Public Declare Function WriteFile Lib "kernel32" (ByVal hFile As Long, lpBuffer As Any, ByVal nNumberOfBytesToWrite As Long, lpNumberOfBytesWritten As Long, ByVal lpOverlapped As Long) As Long
Public Declare Function ReadFile Lib "kernel32" (ByVal hFile As Long, lpBuffer As Any, ByVal nNumberOfBytesToRead As Long, lpNumberOfBytesRead As Long, lpOverlapped As Long) As Long
Public Declare Function CreateFile Lib "kernel32" Alias "CreateFileA" (ByVal lpFileName As String, ByVal dwDesiredAccess As Long, ByVal dwShareMode As Long, ByVal lpSecurityAttributes As Long, ByVal dwCreationDisposition As Long, ByVal dwFlagsAndAttributes As Long, ByVal hTemplateFile As Long) As Long
Public Declare Function SetFilePointer Lib "kernel32.dll" (ByVal hFile As Long, ByVal lDistanceToMove As Long, lpDistanceToMoveHigh As Long, ByVal dwMoveMethod As Long) As Long

'----------------------------------------------------------------------------------------------------------------
'--- this function writes RAW data to a specified drive-letter and a specified sector (512 bytes per sector)  ---
'----------------------------------------------------------------------------------------------------------------
Public Function WriteRawData(StrData As String, StrDriveLetter As String, LngSector As Long) As Boolean
    '--- declares ---
    Dim LngDriveHandle As Long
    Dim LngFilePointerHandle As Long
    Dim LngBytesWritten As Long
    
    '--- create handle for device ---
    LngDriveHandle = CreateFile("\\.\" & StrDriveLetter & ":", GENERIC_WRITE, FILE_SHARE_READ Or FILE_SHARE_WRITE, ByVal 0&, OPEN_EXISTING, 0, ByVal 0&)
    
    '--- check if we have a valid drive handle ---
    If LngDriveHandle <> INVALID_HANDLE_VALUE Then
        LngFilePointerHandle = SetFilePointer(LngDriveHandle, LngSector * 512, 0, 0)
        '--- filepointerhandle is allways the same as the requested sector * 512 ---
        If LngFilePointerHandle <> LngSector * 512 Then
            WriteRawData = False
            Exit Function
        End If
        
        '--- if data is to short, then add spaces, so we have 512 bytes ---
        StrData = StrData & Space(520)
        StrData = Left(StrData, 512)
        
        WriteRawData = WriteFile(LngDriveHandle, ByVal (StrData), 512, LngBytesWritten, ByVal 0&)
        '--- number of bytes written should equal 512, otherwise something is wrong ---
        If LngBytesWritten = 512 Then
            WriteRawData = True
        Else
            WriteRawData = False
        End If
        '--- release handles ---
        CloseHandle LngFilePointerHandle
        CloseHandle LngDriveHandle
    Else
        '--- invalid drive handle ---
        WriteRawData = False
    End If
End Function

'----------------------------------------------------------------------------------------------------------------
'--- this function reads RAW data from a specified drive-letter and a specified sector (512 bytes per sector) ---
'----------------------------------------------------------------------------------------------------------------
Public Function ReadRawData(StrDriveLetter As String, LngSector As Long) As String
    '--- declares ---
    Dim LngDriveHandle As Long
    Dim LngFilePointerHandle As Long
    Dim LngValue As Long
    Dim LngBytesRead As Long
    Dim strBuffer(1 To 512) As Byte
    
    '--- create handle for device ---
    LngDriveHandle = CreateFile("\\.\" & StrDriveLetter & ":", GENERIC_READ, FILE_SHARE_READ Or FILE_SHARE_WRITE, ByVal 0&, OPEN_EXISTING, 0, ByVal 0&)
    
    '--- check if we have a valid drive handle ---
    If LngDriveHandle <> INVALID_HANDLE_VALUE Then
        LngFilePointerHandle = SetFilePointer(LngDriveHandle, LngSector * 512, 0, 0)
        '--- filepointerhandle is allways the same as the requested sector * 512 ---
        If LngFilePointerHandle <> LngSector * 512 Then
            ReadRawData = ""
            Exit Function
        End If
        LngValue = ReadFile(LngDriveHandle, strBuffer(1), 512, LngBytesRead, ByVal 0&)
        '--- number of bytes read should equal 512, otherwise something is wrong ---
        If LngBytesRead = 512 Then
            ReadRawData = ByteArrayToString(strBuffer())
        Else
            ReadRawData = ""
        End If
        '--- release handles ---
        CloseHandle LngFilePointerHandle
        CloseHandle LngFileHandle
    Else
        '--- invalid drive handle ---
        ReadRawData = ""
    End If
    
End Function

'----------------------------------------------------------------------------------------------------------------
'--- this function converts all bytes in an array to a string-variable                                        ---
'----------------------------------------------------------------------------------------------------------------
Private Function ByteArrayToString(bytArray() As Byte) As String
    '--- declares ---
    Dim StrAns As String
    Dim IntPos As String
    
    StrAns = StrConv(bytArray, vbUnicode)
    IntPos = InStr(StrAns, Chr(0))
    If IntPos > 0 Then StrAns = Left(StrAns, IntPos - 1)
    
    ByteArrayToString = StrAns
 End Function
 
'----------------------------------------------------------------------------------------------------------------
'--- this function returns the number of sectors for the given drive                                          ---
'----------------------------------------------------------------------------------------------------------------
Public Function GetSectorsForDrive(StrDriveLetter As String) As Long
    '--- declares ---
    Dim LngDriveHandle As Long
    Dim tPart As PARTITION_INFORMATION
    Dim ovrTemp As OVERLAPPED
    Dim bytesReturned As Long
 
    '--- create drive handle ---
    LngDriveHandle = CreateFile("\\.\" & StrDriveLetter & ":", GENERIC_READ, FILE_SHARE_READ Or FILE_SHARE_WRITE, ByVal 0&, OPEN_EXISTING, 0, ByVal 0&)
    
    '--- check if we have a valid drive handle ---
    If LngDriveHandle = INVALID_HANDLE_VALUE Then
        GetSectorsForDrive = -1
        Exit Function
    End If
    
    '--- get the actual number of sectors ---
    Call DeviceIoControl(LngDriveHandle, IOCTL_DISK_GET_PARTITION_INFO, ByVal 0&, 0&, _
        tPart, Len(tPart), bytesReturned, ovrTemp)
    
    GetSectorsForDrive = ((tPart.PartitionLength * 10000) / 512)
    
    '--- close drive handle ---
    Call CloseHandle(LngDriveHandle)
End Function
 
'----------------------------------------------------------------------------------------------------------------
'--- this function returns the type of drive, for the given driveletter                                       ---
'---            Unknown         = 0                                                                           ---
'---            Removable Drive = 2 (this type of device is used in this application !)                       ---
'---            Fixed Drive     = 3                                                                           ---
'---            DriveShare      = 4                                                                           ---
'---            CDROM           = 5                                                                           ---
'----------------------------------------------------------------------------------------------------------------
 Public Function GetDriveType(StrDriveLetter As String) As Long
    GetDriveType = GetDriveTypeAPI(StrDriveLetter & ":\")
 End Function

'----------------------------------------------------------------------------------------------------------------
'--- return a string with all valid drive-letters for this machine                                            ---
'----------------------------------------------------------------------------------------------------------------
Public Function GetDriveString() As String
   '--- declares ---
   Dim StrBuff As String
   
   StrBuff = Space$((26 * 4) + 1)
   If GetLogicalDriveStrings(Len(StrBuff), StrBuff) Then
     'just trim off the trailing spaces - leave the nulls
      GetDriveString = Trim$(StrBuff)
   End If

End Function

'-------------------------------------------------------------------------------------------------------------------
'--- this function is used as a sub-function of GetDriveString, to split all drive-letters into separate strings ---
'-------------------------------------------------------------------------------------------------------------------
Public Function StripNulls(Strstart As String) As String
   '--- declares ---
   Dim LngPos As Long

   LngPos = InStr(Strstart$, Chr$(0))
   If LngPos Then
      StripNulls = Mid$(Strstart, 1, LngPos - 1)
      Strstart = Mid$(Strstart, LngPos + 1, Len(Strstart))
   End If

End Function

'-------------------------------------------------------------------------------------------------------------------
'--- this function checks on the given memorycard if we have enough space for an extra animation,                ---
'--- with the given resolution and number of frames                                                              ---
'-------------------------------------------------------------------------------------------------------------------
Public Function CheckEnoughFreeSpace(IntHorizontal As Integer, IntVertical As Integer, LngNumberOfFrames As Long, StrDrive As String) As Boolean
    '--- there need to be at least 2 extra sectors free                                                   ---
    '--- one for the entry in the files section, and one possibility to add a sequence with the animation ---
    If GetFreeSectorsForDrive(StrDrive) + 2 >= SectorsNeeded(IntHorizontal, IntVertical, LngNumberOfFrames) Then
        CheckEnoughFreeSpace = True
    Else
        CheckEnoughFreeSpace = False
    End If
    
End Function

'-------------------------------------------------------------------------------------------------------------------
'--- this function returns the number of sectors needed for a given resolution & number of frames                ---
'-------------------------------------------------------------------------------------------------------------------
Public Function SectorsNeeded(IntHorizontal As Integer, IntVertical As Integer, LngNumberOfFrames As Long) As Long
    '--- declares ---
    Dim LngBytesPerFrame As Long

    '--- first, calculate how many sectors are needed ---
    LngBytesPerFrame = IntHorizontal * IntVertical
    SectorsNeeded = Int((LngBytesPerFrame / 512) + 0.9999) * LngNumberOfFrames

End Function

'-------------------------------------------------------------------------------------------------------------------
'--- this function returns the number of free sectors on the given memorycard                                    ---
'-------------------------------------------------------------------------------------------------------------------
Public Function GetFreeSectorsForDrive(StrDrive As String) As Long
    '--- declares ---
    Dim LngUpperSector As Long
    Dim LngLowerSector As Long
    Dim LngI As Long
    Dim BoolEnd As Boolean
    Dim StrData As String
    
    LngI = GetSectorsForDrive(StrDrive) - 1
    LngUpperSector = LngI - 1
    LngLowerSector = 1
    
    BoolEnd = False
    While BoolEnd = False And LngI > 1
        LngI = LngI - 1
        StrData = ReadRawData(StrDrive, LngI)
        If Left(StrData, 8) <> "[GLYPHIX" Then
            BoolEnd = True
        Else
            LngUpperSector = LngI
            If Left(StrData, 12) = "[GLYPHIX FX]" Then
                LngLowerSector = CLng(Mid$(StrData, 93, 10))
            End If
        End If
    Wend
    
    '--- now that we have the upper and lower sector, we can simple extract them from each other ---
    GetFreeSectorsForDrive = LngUpperSector - LngLowerSector + 1
End Function

'-------------------------------------------------------------------------------------------------------------------
'--- this function returns the first free lower sector                                                           ---
'-------------------------------------------------------------------------------------------------------------------
Public Function GetFirstFreeLowerSector(StrDrive As String) As Long
    '--- declares ---
    Dim LngI As Long
    Dim BoolEnd As Boolean
    Dim StrData As String
    
    If GetFreeSectorsForDrive(StrDrive) > 1 Then
        BoolEnd = False
        LngI = GetSectorsForDrive(StrDrive) - 1
        GetFirstFreeLowerSector = 1
        While BoolEnd = False And LngI > 1
            LngI = LngI - 1
            StrData = ReadRawData(StrDrive, LngI)
            If Left(StrData, 8) <> "[GLYPHIX" Then
                BoolEnd = True
            Else
                If Left(StrData, 12) = "[GLYPHIX FX]" Or Left(StrData, 12) = "[GLYPHIX*FX]" Then
                    GetFirstFreeLowerSector = CLng(Mid$(StrData, 93, 10)) + 1
                End If
            End If
        Wend
    Else
        GetFirstFreeLowerSector = -1
    End If
End Function

'-------------------------------------------------------------------------------------------------------------------
'--- this function returns the first free upper sector                                                           ---
'-------------------------------------------------------------------------------------------------------------------
Public Function GetFirstFreeUpperSector(StrDrive As String) As Long
    '--- declares ---
    Dim LngI As Long
    Dim BoolEnd As Boolean
    
    If GetFreeSectorsForDrive(StrDrive) > 1 Then
        BoolEnd = False
        LngI = GetSectorsForDrive(StrDrive) - 1
        While BoolEnd = False And LngI > 1
            LngI = LngI - 1
            If Left(ReadRawData(StrDrive, LngI), 8) <> "[GLYPHIX" Then
                BoolEnd = True
            End If
        Wend
        GetFirstFreeUpperSector = LngI
    Else
        GetFirstFreeUpperSector = -1
    End If
    
End Function

'-------------------------------------------------------------------------------------
'--- this function checks if there is a valid project on the selected drive-letter ---
'-------------------------------------------------------------------------------------
Public Function CheckForValidProject(StrDriveLetter As String) As Boolean
    '--- declares ---
    Dim LngNumberOfSectors As Long
    Dim StrSectorString As String
    
    LngNumberOfSectors = GetSectorsForDrive(StrDriveLetter)
    
    StrSectorString = ReadRawData(StrDriveLetter, LngNumberOfSectors - 1)
    If StrSectorString = gAppSignature Then
        CheckForValidProject = True
    Else
        CheckForValidProject = False
    End If
End Function

'-------------------------------------------------------------------------------------
'--- this function returns the filename of a string containing path & filename     ---
'-------------------------------------------------------------------------------------
Public Function GetFileName(StrData As String) As String
    '--- declares ---
    Dim IntI As Integer
    
    StrData = Trim(StrData)
    For IntI = Len(StrData) To 1 Step -1
        If Mid$(StrData, IntI, 1) = "\" Then
            GetFileName = Mid$(StrData, IntI + 1, Len(StrData) - IntI)
            Exit Function
        End If
    Next IntI

End Function

'-------------------------------------------------------------------------------------
'--- this function returns the path of a string, containing path & filename        ---
'-------------------------------------------------------------------------------------
Public Function GetPath(StrData As String) As String
    '--- declares ---
    Dim IntI As Integer

    StrData = Trim(StrData)
    For IntI = Len(StrData) To 1 Step -1
        If Mid$(StrData, IntI, 1) = "\" Then
            GetPath = Left(StrData, IntI)
            Exit Function
        End If
    Next IntI

End Function
